﻿Type=Class
Version=3
ModulesStructureVersion=1
B4i=true
@EndOfDesignText@
'version 1.00
Sub Class_Globals
	Private mTarget As Object
	Private mEvent As String
	Private link As String
	Private const VERSION As Float = 1
End Sub

'Target - The module that handles JobDone (usually Me).
'ConnectorLink - URL of the Java server.
Public Sub Initialize (Target As Object, EventName As String, ConnectorLink As String)
	mTarget = Target
	link = ConnectorLink
	mEvent = EventName
End Sub

'Inserts a single document.
'The NonFind event will be raised.
Public Sub InsertOne (Collection As String, Document As Map, Tag As Object)
	InsertMany (Collection, Array (Document), Tag)
End Sub

'Inserts one or more documents.
'The NonFind event will be raised.
Public Sub InsertMany (Collection As String, Documents As List, Tag As Object)
	SendJob (CreateCommand(Collection, "Insert", Documents), Tag)
End Sub

'Replaces an existing document or adds a new document if not found.
'The NonFind event will be raised.
Public Sub InsertOrReplace (Collection As String, Filter As Map, Document As Map, Tag As Object)
	SendJob (CreateCommand(Collection, "Upsert", Array(Filter, Document)), Tag)
End Sub

Private Sub CreateCommand (Collection As String, Method As String, Maps As List) As MDBCommand
	Dim cmd11 As MDBCommand
	cmd11.Initialize
	cmd11.Method = Method
	cmd11.Collection = Collection
	cmd11.Maps = Maps
	Return cmd11
End Sub

'Returns the matching documents.
'The Find event will be raised.
Public Sub Find (Collection As String, Filter As Map, Tag As Object)
	Find2 (Collection, Filter, CreateMap(), 0, Tag)
End Sub

'Returns the matching documents. Allows setting the sort map and limit.
'The Find event will be raised.
Public Sub Find2 (Collection As String, Filter As Map, Sort As Map, Limit As Int, Tag As Object)
	Dim cmd1 As MDBCommand = CreateCommand (Collection, "Find", Array (Filter, Sort))
	cmd1.Limit = Limit
	SendJob(cmd1, Tag)
	
End Sub

'Deletes documents from the collection.
'The NonFind event will be raised.
Public Sub Delete (Collection As String, Filter As Map, Tag As Object)
	SendJob(CreateCommand(Collection, "Delete", Array(Filter)), Tag)
End Sub

Public Sub Aggregate (Collection As String, Pipeline As List, Tag As Object)
	SendJob (CreateCommand(Collection, "Aggregate", Pipeline), Tag)
End Sub

Private Sub SendJob (cmd1 As MDBCommand, Tag As Object)
	Dim j As HttpJob
	j.Initialize("MDBRequest_" & cmd1.Method, mTarget)
	j.Tag = Tag
	Dim ser As B4XSerializator
	j.PostBytes($"${link}?$1.2{VERSION}"$, ser.ConvertObjectToBytes(cmd1))
End Sub

'Should be called from JobDone. Returns True if the job was handled.
Public Sub HandleJob (job As HttpJob) As Boolean
	If job.JobName.StartsWith("MDBRequest") = False Then Return False
	Dim res As MDBResult
	res.Initialize
	res.Success = job.Success
	res.Tag = job.Tag
	If job.JobName.EndsWith("Find") Or job.JobName.EndsWith("Aggregate") Then
		If job.Success = False Then
			CallSub3(mTarget, mEvent & "_FindResult", res, Null)
		Else
			Dim ser As B4XSerializator
			ser.Tag = res
			ser.ConvertBytesToObjectAsync(Bit.InputStreamToBytes(job.GetInputStream), "ser")
		End If
	Else
		CallSub2(mTarget, mEvent & "_NonFindResult", res)
	End If
	job.Release
	Return True
End Sub

Private Sub ser_BytesToObject (Success As Boolean, NewObject As Object)
	Dim ser As B4XSerializator = Sender
	Dim res As MDBResult = ser.Tag
	res.Success = Success
	Dim documents As List
	If Success Then documents = NewObject
	CallSub3(mTarget, mEvent & "_FindResult", res, documents)
End Sub

